// This file is part of the PiDuce project (http://www.cs.unibo.it/Piduce/)
// developed at the Department of Computer Science of Bologna.
// Authors: Samuele Carpineti, Cosimo Laneve, Luca Padovani
//
// See Copyright for the status of this software.

using System;
using System.Xml;
using PiDuce.Web;
using System.Threading;
using System.Collections;

namespace PiDuce.Machine
{
  class MachineOptions 
  {
    private int port = 1811;
		private string host = null;
    private bool keepAlive = false;
    private bool usage = false;
    private IList programs = new ArrayList();

    public int Port
    { 
      get { return port; }
      set { port = value; }
    }
		public string Host
		{
			get { return host; }
			set { host = value; }
		}
    public bool KeepAlive
    { 
      get { return keepAlive; }
      set { keepAlive = value; }
    }
    public bool Usage
    {
      get { return usage; } 
      set { usage = value; } 
    }
    public IList Programs { get { return programs; } }
  }

	public class Machine
	{
    private static string usage = "Usage: machine.exe [OPTION] [FILE] ... " 
      + System.Environment.NewLine 
      + "Run a PiDuce machine and execute FILEs"
      + System.Environment.NewLine + "Options (use \"/\" on Windows!)" + System.Environment.NewLine
      + "-port:PORT, /port:PORT" + "\t use PORT instead of the default port number 1811" 
      + System.Environment.NewLine
			+ "-host:ADDR, /host:ADDR" + "\t force the host name/address to ADDR"
      + System.Environment.NewLine
      + "-d, /d" + "\t keep the machine available over the network"
      + System.Environment.NewLine;

    public static string Usage { get { return usage; } } 
    
    private static MachineOptions Parse(string[] args)
    {
      MachineOptions options = new MachineOptions();
      
      foreach (string par in args) //parsing arguments
      {
        if (par.StartsWith("-port:") || par.StartsWith("/port:"))
        { 
          try 
          { options.Port = Int16.Parse(par.Substring("-port".Length + 1)); }
          catch (OverflowException) 
          { Console.WriteLine("Wrong port number ... using the default port number {0}", options.Port); }
        }
				else if (par.StartsWith("-host:") || par.StartsWith("/host:"))
				{ options.Host = par.Substring("-host".Length + 1); }
        else if (par.StartsWith("-d") || par.StartsWith("/d"))
        { options.KeepAlive = true; }
        else if (par.StartsWith("-help") || par.StartsWith("/help"))
        { options.Usage = true; }
        else 
        { 
          try 
          {
            XmlDocument program = new XmlDocument();
            program.Load(par);
            options.Programs.Add(program);
          } 
          catch (Exception e)
          { Console.WriteLine("Error loading the program {0} {1}", par, e);  }
        }
      }
      return options;
    }

		[STAThread]
		static void Main(string[] args)
    {	
      MachineOptions options = Parse(args);
      if (options.Usage) 
      { Console.WriteLine(Usage); }
			if (options.Host == null) {
				options.Host = System.Net.Dns.GetHostName();
				Console.WriteLine("Host name not specified, taking default: " + options.Host); 
			}

      //starting the machine
      ChannelManager chmgr = new ChannelManager(); 
      Scheduler scheduler = new Scheduler(chmgr, options.KeepAlive);
      foreach (XmlDocument bytecode in options.Programs)
      { scheduler.AddProgram(bytecode.InnerXml);  }

      WebServer.Init(chmgr, scheduler, options.Host, options.Port);
      Thread wThread = new Thread(new ThreadStart(WebServer.StartListening));
      wThread.Name = "WebServer";
      wThread.Start();
      scheduler.Run();
		}
	}
}
