// This file is part of the PiDuce project (http://www.cs.unibo.it/Piduce/)
// developed at the Department of Computer Science of Bologna.
// Authors: Samuele Carpineti, Cosimo Laneve, Luca Padovani
//
// See Copyright for the status of this software.

using System;
using System.Collections;
using System.Diagnostics;
using PiDuce.Common;

namespace PiDuce.Types 
{
	public class TopLevelEntriesVisitor : AbstractSchemaVisitor 
	{
		private	ArraySet<IEntry> head;

		public ArraySet<IEntry> GetEntries(ISchema s)
		{
			head = new ArraySet<IEntry>();
			s.Accept(this);
			return head;
		}

		public override void VisitConstantSchema(ConstantSchema s)
		{
			if (!head.Contains(s.Entry)) {
				head.Add(s.Entry);
				Debug.Assert(s.Entry != null && s.Entry.Schema != null);
				s.Entry.Schema.Accept(this);
			}
		}

		public override void VisitChannelSchema(ChannelSchema s)
		{ }

		public override void VisitFunctionSchema(FunctionSchema s)
		{ }

		public override void VisitRecordSchema(RecordSchema s)
		{ }
	}

	public class CorrectnessChecker
	{
    private readonly DeterminednessChecker dc;
		private readonly TopLevelEntriesVisitor ev = new TopLevelEntriesVisitor();

    public CorrectnessChecker(DeterminednessChecker dc)
    { this.dc = dc; }
    
    public bool IsGuarded(ConstantSchema s)
    {
			Debug.Assert(s.Entry != null && s.Entry.Schema != null);
			ArraySet<IEntry> head = ev.GetEntries(s.Entry.Schema);
			return !head.Contains(s.Entry); 
    }
    
		public bool IsCorrect(ISchema s, out string error)
		{
            /// <summary>
            /// A schema is correct if and only if both the following conditions hold: 
            /// 1. every recursion is guarded (e.g. A = A and A = B when B = A are not correct)
            /// 2. the content of every channel schema is determined
            /// </summary>
			// TO BE IMPLEMENTED
			error = "yeah, right";
			return true;
		}	
	}
}
