// This file is part of the PiDuce project (http://www.cs.unibo.it/Piduce/)
// developed at the Department of Computer Science of Bologna.
// Authors: Samuele Carpineti, Cosimo Laneve, Luca Padovani
//
// See Copyright for the status of this software.

using System;
using System.Collections.Generic;
using System.Diagnostics;
using PiDuce.Common;

namespace PiDuce.Types
{
	public class EmptynessChecker : ISchemaVisitor
	{
		private IList<ConstantSchema> visited;
		private bool result;

		public EmptynessChecker()
		{
			visited = new List<ConstantSchema>();
		}

		public bool IsEmpty(ISchema s)
		{
			s.Accept(this);
			return result;
		}

		public void VisitVoidSchema(VoidSchema s)
		{ result = false; }

		public void VisitBasicSchema(BasicSchema s)
		{ result = false; }

		public void VisitBasicSchemaLiteralSchema(BasicSchemaLiteralSchema s)
		{ result = false; }

		public void VisitLabelledSchema(LabelledSchema s)
		{ result = s.Labels.IsEmpty() || IsEmpty(s.Content); }

		public void VisitSequenceSchema(SequenceSchema s)
		{ result = IsEmpty(s.Head) || IsEmpty(s.Tail); }

		public void VisitUnionSchema(UnionSchema s)
		{ result = IsEmpty(s.Left) && IsEmpty(s.Right); }

		public void VisitStarSchema(StarSchema s)
		{ result = false; }

		public void VisitPlusSchema(PlusSchema s)
		{ result = IsEmpty(s.Content); }

		public void VisitRepetitionSchema(RepetitionSchema s)
		{ result = s.MinOccurs > 0 && IsEmpty(s.Content); }

		public void VisitConstantSchema(ConstantSchema s)
		{
			if (visited.Contains(s))
				result = true;
			else {
				Debug.Assert(s.Entry != null);
				Debug.Assert(s.Entry.Schema != null);
				visited.Add(s);
				result = IsEmpty(s.Entry.Schema);
				visited.Remove(s);
			}	
		}

		public void VisitChannelSchema(ChannelSchema s)
		{ result = false; }

		public void VisitFunctionSchema(FunctionSchema s)
		{ result = false; }

		public void VisitErrorSchema(ErrorSchema s)
		{ result = false; }

		public void VisitBindSchema(BindSchema s)
		{ result = IsEmpty(s.Content); }

		public void VisitRecordSchema(RecordSchema s)
		{ result = false; } // WATCH ME
	}
}
